#include <stdio.h>
#include <stdlib.h>

typedef struct dll dll_t;
typedef struct dll_item dll_item_t;


// Double linked list with head
struct dll {
    dll_item_t      *head;
};

struct dll_item {
    dll_item_t      *next;
    dll_item_t      *prev;
};

#define DLL_NULL ((dll_item_t *) 0)

#define DLL_DIE do { \
    /* this call should never return */ \
    dll_die(__FUNCTION__); \
    abort(); \
} while (0)

#define DLL_NEW(type) \
    ((type *) malloc(sizeof(type)))


void dll_die(const char *msg)
{
    (void) msg;
    abort();
}

static dll_item_t* create_item(void)
{
    dll_item_t *item = DLL_NEW(dll_item_t);
    if (!item)
        DLL_DIE;

    item->next = NULL;
    item->prev = NULL;

    return item;
}


dll_t* dll_init(dll_t *list)
{
    list->head = create_item();
}

dll_item_t* dll_end(dll_t *list)
{
    return list->head->next;
}

dll_item_t* dll_next(dll_item_t *item)
{
    return item->next;
}

dll_item_t* dll_prev(dll_item_t *item)
{
    return item->prev;
}

dll_item_t* dll_push_back(dll_t *list)
{
    dll_item_t *item = create_item();
    if ((item->prev = list->head->prev))
        item->prev->next = item;
    else
        list->head->next = item;
    list->head->prev = item;
    return item;
}

void dll_remove(dll_t *list, dll_item_t *item)
{
    if (!item)
        return;

    if (item->next)
        item->next->prev = item->prev;
    else
        list->head->prev = item->prev;


    if (item->prev)
        item->prev->next = item->next;
    else
        list->head->next = item->next;

    free(item);
}

int main()
{
    dll_t* list = (dll_t*)malloc(sizeof(dll_t));
    dll_init(list);

    dll_push_back(list);
    dll_push_back(list);
    dll_push_back(list);

    dll_remove(list, dll_prev(dll_end(list)));


    while(dll_end(list) != NULL) {
        dll_remove(list, dll_end(list));
    }


    free(list);
    return 0;
}
